@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2004-2010 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2004-2017 Erik de Castro Lopo <erikd@mega-nerd.com>
 ** Copyright (C) 2004 Tobias Gehrig <tgehrig@ira.uka.de>
 **
 ** This program is free software ; you can redistribute it and/or modify
@@ -29,7 +29,7 @@
 #include	"sndfile.h"
 #include	"common.h"
 
-#if HAVE_EXTERNAL_LIBS
+#if HAVE_EXTERNAL_XIPH_LIBS
 
 #include	<FLAC/stream_decoder.h>
 #include	<FLAC/stream_encoder.h>
@@ -39,6 +39,8 @@
 ** Private static functions.
 */
 
+#define	FLAC_DEFAULT_COMPRESSION_LEVEL	5
+
 #define ENC_BUFFER_SIZE 8192
 
 typedef enum
@@ -59,14 +61,16 @@ typedef struct
 
 	FLAC__StreamMetadata *metadata ;
 
-	const FLAC__int32 * const * wbuffer ;
-	FLAC__int32 * rbuffer [FLAC__MAX_CHANNELS] ;
+	const int32_t * const * wbuffer ;
+	int32_t * rbuffer [FLAC__MAX_CHANNELS] ;
 
-	FLAC__int32* encbuffer ;
+	int32_t* encbuffer ;
 	unsigned bufferpos ;
 
 	const FLAC__Frame *frame ;
-	FLAC__bool bufferbackup ;
+
+	unsigned compression ;
+
 } FLAC_PRIVATE ;
 
 typedef struct
@@ -75,6 +79,7 @@ typedef struct
 } FLAC_TAG ;
 
 static sf_count_t	flac_seek (SF_PRIVATE *psf, int mode, sf_count_t offset) ;
+static int			flac_byterate (SF_PRIVATE *psf) ;
 static int			flac_close (SF_PRIVATE *psf) ;
 
 static int			flac_enc_init (SF_PRIVATE *psf) ;
@@ -90,18 +95,18 @@ static sf_count_t	flac_write_i2flac (SF_PRIVATE *psf, const int *ptr, sf_count_t
 static sf_count_t	flac_write_f2flac (SF_PRIVATE *psf, const float *ptr, sf_count_t len) ;
 static sf_count_t	flac_write_d2flac (SF_PRIVATE *psf, const double *ptr, sf_count_t len) ;
 
-static void		f2flac8_array (const float *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		f2flac16_array (const float *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		f2flac24_array (const float *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		f2flac8_clip_array (const float *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		f2flac16_clip_array (const float *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		f2flac24_clip_array (const float *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		d2flac8_array (const double *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		d2flac16_array (const double *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		d2flac24_array (const double *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		d2flac8_clip_array (const double *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		d2flac16_clip_array (const double *src, FLAC__int32 *dest, int count, int normalize) ;
-static void		d2flac24_clip_array (const double *src, FLAC__int32 *dest, int count, int normalize) ;
+static void		f2flac8_array (const float *src, int32_t *dest, int count, int normalize) ;
+static void		f2flac16_array (const float *src, int32_t *dest, int count, int normalize) ;
+static void		f2flac24_array (const float *src, int32_t *dest, int count, int normalize) ;
+static void		f2flac8_clip_array (const float *src, int32_t *dest, int count, int normalize) ;
+static void		f2flac16_clip_array (const float *src, int32_t *dest, int count, int normalize) ;
+static void		f2flac24_clip_array (const float *src, int32_t *dest, int count, int normalize) ;
+static void		d2flac8_array (const double *src, int32_t *dest, int count, int normalize) ;
+static void		d2flac16_array (const double *src, int32_t *dest, int count, int normalize) ;
+static void		d2flac24_array (const double *src, int32_t *dest, int count, int normalize) ;
+static void		d2flac8_clip_array (const double *src, int32_t *dest, int count, int normalize) ;
+static void		d2flac16_clip_array (const double *src, int32_t *dest, int count, int normalize) ;
+static void		d2flac24_clip_array (const double *src, int32_t *dest, int count, int normalize) ;
 
 static int flac_command (SF_PRIVATE *psf, int command, void *data, int datasize) ;
 
@@ -111,7 +116,7 @@ static FLAC__StreamDecoderSeekStatus sf_flac_seek_callback (const FLAC__StreamDe
 static FLAC__StreamDecoderTellStatus sf_flac_tell_callback (const FLAC__StreamDecoder *decoder, FLAC__uint64 *absolute_byte_offset, void *client_data) ;
 static FLAC__StreamDecoderLengthStatus sf_flac_length_callback (const FLAC__StreamDecoder *decoder, FLAC__uint64 *stream_length, void *client_data) ;
 static FLAC__bool sf_flac_eof_callback (const FLAC__StreamDecoder *decoder, void *client_data) ;
-static FLAC__StreamDecoderWriteStatus sf_flac_write_callback (const FLAC__StreamDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 * const buffer [], void *client_data) ;
+static FLAC__StreamDecoderWriteStatus sf_flac_write_callback (const FLAC__StreamDecoder *decoder, const FLAC__Frame *frame, const int32_t * const buffer [], void *client_data) ;
 static void sf_flac_meta_callback (const FLAC__StreamDecoder *decoder, const FLAC__StreamMetadata *metadata, void *client_data) ;
 static void sf_flac_error_callback (const FLAC__StreamDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data) ;
 
@@ -121,38 +126,38 @@ static FLAC__StreamEncoderTellStatus sf_flac_enc_tell_callback (const FLAC__Stre
 static FLAC__StreamEncoderWriteStatus sf_flac_enc_write_callback (const FLAC__StreamEncoder *encoder, const FLAC__byte buffer [], size_t bytes, unsigned samples, unsigned current_frame, void *client_data) ;
 
 static void
-s2flac8_array (const short *src, FLAC__int32 *dest, int count)
+s2flac8_array (const short *src, int32_t *dest, int count)
 {	while (--count >= 0)
 		dest [count] = src [count] >> 8 ;
 } /* s2flac8_array */
 
 static void
-s2flac16_array (const short *src, FLAC__int32 *dest, int count)
+s2flac16_array (const short *src, int32_t *dest, int count)
 {	while (--count >= 0)
 		dest [count] = src [count] ;
 } /* s2flac16_array */
 
 static void
-s2flac24_array (const short *src, FLAC__int32 *dest, int count)
+s2flac24_array (const short *src, int32_t *dest, int count)
 {	while (--count >= 0)
 		dest [count] = src [count] << 8 ;
 } /* s2flac24_array */
 
 static void
-i2flac8_array (const int *src, FLAC__int32 *dest, int count)
+i2flac8_array (const int *src, int32_t *dest, int count)
 {	while (--count >= 0)
 		dest [count] = src [count] >> 24 ;
 } /* i2flac8_array */
 
 static void
-i2flac16_array (const int *src, FLAC__int32 *dest, int count)
+i2flac16_array (const int *src, int32_t *dest, int count)
 {
-  while (--count >= 0)
-    dest [count] = src [count] >> 16 ;
+	while (--count >= 0)
+		dest [count] = src [count] >> 16 ;
 } /* i2flac16_array */
 
 static void
-i2flac24_array (const int *src, FLAC__int32 *dest, int count)
+i2flac24_array (const int *src, int32_t *dest, int count)
 {	while (--count >= 0)
 		dest [count] = src [count] >> 8 ;
 } /* i2flac24_array */
@@ -161,8 +166,8 @@ static sf_count_t
 flac_buffer_copy (SF_PRIVATE *psf)
 {	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
 	const FLAC__Frame *frame = pflac->frame ;
-	const FLAC__int32* const *buffer = pflac->wbuffer ;
-	unsigned i = 0, j, offset ;
+	const int32_t* const *buffer = pflac->wbuffer ;
+	unsigned i = 0, j, offset, channels, len ;
 
 	/*
 	**	frame->header.blocksize is variable and we're using a constant blocksize
@@ -175,53 +180,71 @@ flac_buffer_copy (SF_PRIVATE *psf)
 		return 0 ;
 		} ;
 
+	if (frame->header.channels > FLAC__MAX_CHANNELS)
+		psf_log_printf (psf, "Ooops : frame->header.channels (%d) > FLAC__MAX_BLOCK_SIZE (%d)\n", __func__, __LINE__, frame->header.channels, FLAC__MAX_CHANNELS) ;
+
+	channels = SF_MIN (frame->header.channels, FLAC__MAX_CHANNELS) ;
+
 	if (pflac->ptr == NULL)
 	{	/*
-		**	Not sure why this code is here and not elsewhere.
-		**	Removing it causes valgrind errors.
+		** This pointer is reset to NULL each time the current frame has been
+		** decoded. Somehow its used during encoding and decoding.
 		*/
-		pflac->bufferbackup = SF_TRUE ;
-		for (i = 0 ; i < frame->header.channels ; i++)
+		for (i = 0 ; i < channels ; i++)
 		{
 			if (pflac->rbuffer [i] == NULL)
-				pflac->rbuffer [i] = calloc (FLAC__MAX_BLOCK_SIZE, sizeof (FLAC__int32)) ;
+				pflac->rbuffer [i] = calloc (FLAC__MAX_BLOCK_SIZE, sizeof (int32_t)) ;
 
-			memcpy (pflac->rbuffer [i], buffer [i], frame->header.blocksize * sizeof (FLAC__int32)) ;
+			memcpy (pflac->rbuffer [i], buffer [i], frame->header.blocksize * sizeof (int32_t)) ;
 			} ;
-		pflac->wbuffer = (const FLAC__int32* const*) pflac->rbuffer ;
+		pflac->wbuffer = (const int32_t* const*) pflac->rbuffer ;
 
 		return 0 ;
 		} ;
 
+
+	len = SF_MIN (pflac->len, frame->header.blocksize) ;
+
+	if (pflac->remain % channels != 0)
+	{	psf_log_printf (psf, "Error: pflac->remain %u    channels %u\n", pflac->remain, channels) ;
+		return 0 ;
+		} ;
+
 	switch (pflac->pcmtype)
 	{	case PFLAC_PCM_SHORT :
 			{	short *retpcm = (short*) pflac->ptr ;
 				int shift = 16 - frame->header.bits_per_sample ;
 				if (shift < 0)
 				{	shift = abs (shift) ;
-					for (i = 0 ; i < frame->header.blocksize && pflac->remain > 0 ; i++)
-					{	offset = pflac->pos + i * frame->header.channels ;
+					for (i = 0 ; i < len && pflac->remain > 0 ; i++)
+					{	offset = pflac->pos + i * channels ;
 
 						if (pflac->bufferpos >= frame->header.blocksize)
 							break ;
 
-						for (j = 0 ; j < frame->header.channels ; j++)
+						if (offset + channels > pflac->len)
+							break ;
+
+						for (j = 0 ; j < channels ; j++)
 							retpcm [offset + j] = buffer [j][pflac->bufferpos] >> shift ;
-						pflac->remain -= frame->header.channels ;
+						pflac->remain -= channels ;
 						pflac->bufferpos++ ;
 						}
 					}
 				else
-				{	for (i = 0 ; i < frame->header.blocksize && pflac->remain > 0 ; i++)
-					{	offset = pflac->pos + i * frame->header.channels ;
+				{	for (i = 0 ; i < len && pflac->remain > 0 ; i++)
+					{	offset = pflac->pos + i * channels ;
 
 						if (pflac->bufferpos >= frame->header.blocksize)
 							break ;
 
-						for (j = 0 ; j < frame->header.channels ; j++)
-							retpcm [offset + j] = (buffer [j][pflac->bufferpos]) << shift ;
+						if (offset + channels > pflac->len)
+							break ;
 
-						pflac->remain -= frame->header.channels ;
+						for (j = 0 ; j < channels ; j++)
+							retpcm [offset + j] = ((uint16_t) buffer [j][pflac->bufferpos]) << shift ;
+
+						pflac->remain -= channels ;
 						pflac->bufferpos++ ;
 						} ;
 					} ;
@@ -231,15 +254,18 @@ flac_buffer_copy (SF_PRIVATE *psf)
 		case PFLAC_PCM_INT :
 			{	int *retpcm = (int*) pflac->ptr ;
 				int shift = 32 - frame->header.bits_per_sample ;
-				for (i = 0 ; i < frame->header.blocksize && pflac->remain > 0 ; i++)
-				{	offset = pflac->pos + i * frame->header.channels ;
+				for (i = 0 ; i < len && pflac->remain > 0 ; i++)
+				{	offset = pflac->pos + i * channels ;
 
 					if (pflac->bufferpos >= frame->header.blocksize)
 						break ;
 
-					for (j = 0 ; j < frame->header.channels ; j++)
-						retpcm [offset + j] = buffer [j][pflac->bufferpos] << shift ;
-					pflac->remain -= frame->header.channels ;
+					if (offset + channels > pflac->len)
+						break ;
+
+					for (j = 0 ; j < channels ; j++)
+						retpcm [offset + j] = ((uint32_t) buffer [j][pflac->bufferpos]) << shift ;
+					pflac->remain -= channels ;
 					pflac->bufferpos++ ;
 					} ;
 				} ;
@@ -249,15 +275,18 @@ flac_buffer_copy (SF_PRIVATE *psf)
 			{	float *retpcm = (float*) pflac->ptr ;
 				float norm = (psf->norm_float == SF_TRUE) ? 1.0 / (1 << (frame->header.bits_per_sample - 1)) : 1.0 ;
 
-				for (i = 0 ; i < frame->header.blocksize && pflac->remain > 0 ; i++)
-				{	offset = pflac->pos + i * frame->header.channels ;
+				for (i = 0 ; i < len && pflac->remain > 0 ; i++)
+				{	offset = pflac->pos + i * channels ;
 
 					if (pflac->bufferpos >= frame->header.blocksize)
 						break ;
 
-					for (j = 0 ; j < frame->header.channels ; j++)
+					if (offset + channels > pflac->len)
+						break ;
+
+					for (j = 0 ; j < channels ; j++)
 						retpcm [offset + j] = buffer [j][pflac->bufferpos] * norm ;
-					pflac->remain -= frame->header.channels ;
+					pflac->remain -= channels ;
 					pflac->bufferpos++ ;
 					} ;
 				} ;
@@ -267,15 +296,18 @@ flac_buffer_copy (SF_PRIVATE *psf)
 			{	double *retpcm = (double*) pflac->ptr ;
 				double norm = (psf->norm_double == SF_TRUE) ? 1.0 / (1 << (frame->header.bits_per_sample - 1)) : 1.0 ;
 
-				for (i = 0 ; i < frame->header.blocksize && pflac->remain > 0 ; i++)
-				{	offset = pflac->pos + i * frame->header.channels ;
+				for (i = 0 ; i < len && pflac->remain > 0 ; i++)
+				{	offset = pflac->pos + i * channels ;
 
 					if (pflac->bufferpos >= frame->header.blocksize)
 						break ;
 
-					for (j = 0 ; j < frame->header.channels ; j++)
+					if (offset + channels > pflac->len)
+						break ;
+
+					for (j = 0 ; j < channels ; j++)
 						retpcm [offset + j] = buffer [j][pflac->bufferpos] * norm ;
-					pflac->remain -= frame->header.channels ;
+					pflac->remain -= channels ;
 					pflac->bufferpos++ ;
 					} ;
 				} ;
@@ -285,8 +317,8 @@ flac_buffer_copy (SF_PRIVATE *psf)
 			return 0 ;
 		} ;
 
-	offset = i * frame->header.channels ;
-	pflac->pos += i * frame->header.channels ;
+	offset = i * channels ;
+	pflac->pos += i * channels ;
 
 	return offset ;
 } /* flac_buffer_copy */
@@ -300,7 +332,7 @@ sf_flac_read_callback (const FLAC__StreamDecoder * UNUSED (decoder), FLAC__byte
 	if (*bytes > 0 && psf->error == 0)
 		return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE ;
 
-    return FLAC__STREAM_DECODER_READ_STATUS_ABORT ;
+	return FLAC__STREAM_DECODER_READ_STATUS_ABORT ;
 } /* sf_flac_read_callback */
 
 static FLAC__StreamDecoderSeekStatus
@@ -342,18 +374,17 @@ sf_flac_eof_callback (const FLAC__StreamDecoder *UNUSED (decoder), void *client_
 	if (psf_ftell (psf) == psf->filelength)
 		return SF_TRUE ;
 
-    return SF_FALSE ;
+	return SF_FALSE ;
 } /* sf_flac_eof_callback */
 
 static FLAC__StreamDecoderWriteStatus
-sf_flac_write_callback (const FLAC__StreamDecoder * UNUSED (decoder), const FLAC__Frame *frame, const FLAC__int32 * const buffer [], void *client_data)
+sf_flac_write_callback (const FLAC__StreamDecoder * UNUSED (decoder), const FLAC__Frame *frame, const int32_t * const buffer [], void *client_data)
 {	SF_PRIVATE *psf = (SF_PRIVATE*) client_data ;
 	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
 
 	pflac->frame = frame ;
 	pflac->bufferpos = 0 ;
 
-	pflac->bufferbackup = SF_FALSE ;
 	pflac->wbuffer = buffer ;
 
 	flac_buffer_copy (psf) ;
@@ -363,7 +394,7 @@ sf_flac_write_callback (const FLAC__StreamDecoder * UNUSED (decoder), const FLAC
 
 static void
 sf_flac_meta_get_vorbiscomments (SF_PRIVATE *psf, const FLAC__StreamMetadata *metadata)
-{	FLAC_TAG tags [] =
+{	static FLAC_TAG tags [] =
 	{ 	{ "title", SF_STR_TITLE },
 		{ "copyright", SF_STR_COPYRIGHT },
 		{ "software", SF_STR_SOFTWARE },
@@ -389,7 +420,7 @@ sf_flac_meta_get_vorbiscomments (SF_PRIVATE *psf, const FLAC__StreamMetadata *me
 		if ((cptr = strchr (value, '=')) != NULL)
 			value = cptr + 1 ;
 
-		psf_log_printf (psf, "  %-10s : %s\n", tags [k].tag, value) ;
+		psf_log_printf (psf, "  %-12s : %s\n", tags [k].tag, value) ;
 		psf_store_string (psf, tags [k].type, value) ;
 		} ;
 
@@ -399,7 +430,8 @@ sf_flac_meta_get_vorbiscomments (SF_PRIVATE *psf, const FLAC__StreamMetadata *me
 static void
 sf_flac_meta_callback (const FLAC__StreamDecoder * UNUSED (decoder), const FLAC__StreamMetadata *metadata, void *client_data)
 {	SF_PRIVATE *psf = (SF_PRIVATE*) client_data ;
-	int bitwidth = 0 ;
+	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
+	int bitwidth = 0, i ;
 
 	switch (metadata->type)
 	{	case FLAC__METADATA_TYPE_STREAMINFO :
@@ -436,6 +468,12 @@ sf_flac_meta_callback (const FLAC__StreamDecoder * UNUSED (decoder), const FLAC_
 
 			if (bitwidth > 0)
 				psf_log_printf (psf, "  Bit width   : %d\n", bitwidth) ;
+
+
+			for (i = 0 ; i < psf->sf.channels ; i++)
+				pflac->rbuffer [i] = calloc (FLAC__MAX_BLOCK_SIZE, sizeof (int32_t)) ;
+
+			pflac->wbuffer = (const int32_t* const*) pflac->rbuffer ;
 			break ;
 
 		case FLAC__METADATA_TYPE_VORBIS_COMMENT :
@@ -504,7 +542,7 @@ sf_flac_enc_seek_callback (const FLAC__StreamEncoder * UNUSED (encoder), FLAC__u
 	if (psf->error)
 		return FLAC__STREAM_ENCODER_SEEK_STATUS_ERROR ;
 
-    return FLAC__STREAM_ENCODER_SEEK_STATUS_OK ;
+	return FLAC__STREAM_ENCODER_SEEK_STATUS_OK ;
 } /* sf_flac_enc_seek_callback */
 
 static FLAC__StreamEncoderTellStatus
@@ -534,7 +572,7 @@ flac_write_strings (SF_PRIVATE *psf, FLAC_PRIVATE* pflac)
 	int	k, string_count = 0 ;
 
 	for (k = 0 ; k < SF_MAX_STRINGS ; k++)
-	{	if (psf->strings [k].type != 0)
+	{	if (psf->strings.data [k].type != 0)
 			string_count ++ ;
 		} ;
 
@@ -546,10 +584,10 @@ flac_write_strings (SF_PRIVATE *psf, FLAC_PRIVATE* pflac)
 		return ;
 		} ;
 
-	for (k = 0 ; k < SF_MAX_STRINGS && psf->strings [k].type != 0 ; k++)
+	for (k = 0 ; k < SF_MAX_STRINGS && psf->strings.data [k].type != 0 ; k++)
 	{	const char * key, * value ;
 
-		switch (psf->strings [k].type)
+		switch (psf->strings.data [k].type)
 		{	case SF_STR_SOFTWARE :
 				key = "software" ;
 				break ;
@@ -584,7 +622,7 @@ flac_write_strings (SF_PRIVATE *psf, FLAC_PRIVATE* pflac)
 				continue ;
 			} ;
 
-		value = psf->strings [k].str ;
+		value = psf->strings.storage + psf->strings.data [k].offset ;
 
 		FLAC__metadata_object_vorbiscomment_entry_from_name_value_pair (&entry, key, value) ;
 		FLAC__metadata_object_vorbiscomment_append_comment (pflac->metadata, entry, /* copy */ SF_FALSE) ;
@@ -612,7 +650,7 @@ flac_write_header (SF_PRIVATE *psf, int UNUSED (calc_length))
 
 	if (psf->error == 0)
 		psf->dataoffset = psf_ftell (psf) ;
-	pflac->encbuffer = calloc (ENC_BUFFER_SIZE, sizeof (FLAC__int32)) ;
+	pflac->encbuffer = calloc (ENC_BUFFER_SIZE, sizeof (int32_t)) ;
 
 	return psf->error ;
 } /* flac_write_header */
@@ -629,6 +667,9 @@ flac_open	(SF_PRIVATE *psf)
 	FLAC_PRIVATE* pflac = calloc (1, sizeof (FLAC_PRIVATE)) ;
 	psf->codec_data = pflac ;
 
+	/* Set the default value here. Over-ridden later if necessary. */
+	pflac->compression = FLAC_DEFAULT_COMPRESSION_LEVEL ;
+
 	if (psf->file.mode == SFM_RDWR)
 		return SFE_BAD_MODE_RW ;
 
@@ -646,24 +687,26 @@ flac_open	(SF_PRIVATE *psf)
 		psf->endian = SF_ENDIAN_BIG ;
 		psf->sf.seekable = 0 ;
 
-		psf->str_flags = SF_STR_ALLOW_START ;
+		psf->strings.flags = SF_STR_ALLOW_START ;
 
 		if ((error = flac_enc_init (psf)))
 			return error ;
 
+		/* In an ideal world we would write the header at this point. Unfortunately
+		** that would prevent string metadata being added so we have to hold off.
+		*/
+
 		psf->write_header = flac_write_header ;
 		} ;
 
 	psf->datalength = psf->filelength ;
 	psf->dataoffset = 0 ;
-	psf->blockwidth = 0 ;
-	psf->bytewidth = 1 ;
 
 	psf->container_close = flac_close ;
 	psf->seek = flac_seek ;
-	psf->command = flac_command ;
+	psf->byterate = flac_byterate ;
 
-	psf->blockwidth = psf->bytewidth * psf->sf.channels ;
+	psf->command = flac_command ;
 
 	switch (subformat)
 	{	case SF_FORMAT_PCM_S8 :	/* 8-bit FLAC.  */
@@ -695,9 +738,7 @@ flac_close	(SF_PRIVATE *psf)
 	if (psf->file.mode == SFM_WRITE)
 	{	FLAC__stream_encoder_finish (pflac->fse) ;
 		FLAC__stream_encoder_delete (pflac->fse) ;
-
-		if (pflac->encbuffer)
-			free (pflac->encbuffer) ;
+		free (pflac->encbuffer) ;
 		} ;
 
 	if (psf->file.mode == SFM_READ)
@@ -724,7 +765,7 @@ flac_enc_init (SF_PRIVATE *psf)
 	**     "FLAC supports linear sample rates from 1Hz - 655350Hz in 1Hz
 	**     increments."
 	*/
-	if ( psf->sf.samplerate < 1 || psf->sf.samplerate > 655350 )
+	if (psf->sf.samplerate < 1 || psf->sf.samplerate > 655350)
 	{	psf_log_printf (psf, "flac sample rate out of range.\n", psf->sf.samplerate) ;
 		return SFE_FLAC_BAD_SAMPLE_RATE ;
 		} ;
@@ -747,6 +788,8 @@ flac_enc_init (SF_PRIVATE *psf)
 			break ;
 		} ;
 
+	if (pflac->fse)
+		FLAC__stream_encoder_delete (pflac->fse) ;
 	if ((pflac->fse = FLAC__stream_encoder_new ()) == NULL)
 		return SFE_FLAC_NEW_DECODER ;
 
@@ -765,6 +808,11 @@ flac_enc_init (SF_PRIVATE *psf)
 		return SFE_FLAC_INIT_DECODER ;
 		} ;
 
+	if (! FLAC__stream_encoder_set_compression_level (pflac->fse, pflac->compression))
+	{	psf_log_printf (psf, "FLAC__stream_encoder_set_compression_level (%d) return false.\n", pflac->compression) ;
+		return SFE_FLAC_INIT_DECODER ;
+		} ;
+
 	return 0 ;
 } /* flac_enc_init */
 
@@ -773,6 +821,8 @@ flac_read_header (SF_PRIVATE *psf)
 {	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
 
 	psf_fseek (psf, 0, SEEK_SET) ;
+	if (pflac->fsd)
+		FLAC__stream_decoder_delete (pflac->fsd) ;
 	if ((pflac->fsd = FLAC__stream_decoder_new ()) == NULL)
 		return SFE_FLAC_NEW_DECODER ;
 
@@ -796,9 +846,37 @@ flac_read_header (SF_PRIVATE *psf)
 } /* flac_read_header */
 
 static int
-flac_command (SF_PRIVATE * UNUSED (psf), int UNUSED (command), void * UNUSED (data), int UNUSED (datasize))
-{
-	return 0 ;
+flac_command (SF_PRIVATE * psf, int command, void * data, int datasize)
+{	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
+	double quality ;
+
+	switch (command)
+	{	case SFC_SET_COMPRESSION_LEVEL :
+			if (data == NULL || datasize != sizeof (double))
+				return SF_FALSE ;
+
+			if (psf->have_written)
+				return SF_FALSE ;
+
+			/* FLAC compression level is in the range [0, 8] while libsndfile takes
+			** values in the range [0.0, 1.0]. Massage the libsndfile value here.
+			*/
+			quality = (*((double *) data)) * 8.0 ;
+			/* Clip range. */
+			pflac->compression = lrint (SF_MAX (0.0, SF_MIN (8.0, quality))) ;
+
+			psf_log_printf (psf, "%s : Setting SFC_SET_COMPRESSION_LEVEL to %u.\n", __func__, pflac->compression) ;
+
+			if (flac_enc_init (psf))
+				return SF_FALSE ;
+
+			return SF_TRUE ;
+
+		default :
+			return SF_FALSE ;
+		} ;
+
+	return SF_FALSE ;
 } /* flac_command */
 
 int
@@ -821,9 +899,6 @@ flac_init (SF_PRIVATE *psf)
 		psf->write_double	= flac_write_d2flac ;
 		} ;
 
-	psf->bytewidth = 1 ;
-	psf->blockwidth = psf->sf.channels ;
-
 	if (psf->filelength > psf->dataoffset)
 		psf->datalength = (psf->dataend) ? psf->dataend - psf->dataoffset : psf->filelength - psf->dataoffset ;
 	else
@@ -835,18 +910,34 @@ flac_init (SF_PRIVATE *psf)
 static unsigned
 flac_read_loop (SF_PRIVATE *psf, unsigned len)
 {	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
+	FLAC__StreamDecoderState state ;
 
 	pflac->pos = 0 ;
 	pflac->len = len ;
 	pflac->remain = len ;
+
+	state = FLAC__stream_decoder_get_state (pflac->fsd) ;
+	if (state > FLAC__STREAM_DECODER_END_OF_STREAM)
+	{	psf_log_printf (psf, "FLAC__stream_decoder_get_state returned %s\n", FLAC__StreamDecoderStateString [state]) ;
+		/* Current frame is busted, so NULL the pointer. */
+		pflac->frame = NULL ;
+		} ;
+
+	/* First copy data that has already been decoded and buffered. */
 	if (pflac->frame != NULL && pflac->bufferpos < pflac->frame->header.blocksize)
 		flac_buffer_copy (psf) ;
 
+	/* Decode some more. */
 	while (pflac->pos < pflac->len)
 	{	if (FLAC__stream_decoder_process_single (pflac->fsd) == 0)
 			break ;
-		if (FLAC__stream_decoder_get_state (pflac->fsd) >= FLAC__STREAM_DECODER_END_OF_STREAM)
+		state = FLAC__stream_decoder_get_state (pflac->fsd) ;
+		if (state >= FLAC__STREAM_DECODER_END_OF_STREAM)
+		{	psf_log_printf (psf, "FLAC__stream_decoder_get_state returned %s\n", FLAC__StreamDecoderStateString [state]) ;
+			/* Current frame is busted, so NULL the pointer. */
+			pflac->frame = NULL ;
 			break ;
+			} ;
 		} ;
 
 	pflac->ptr = NULL ;
@@ -925,6 +1016,7 @@ flac_read_flac2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	while (total < len)
 	{	pflac->ptr = ptr + total ;
 		readlen = (len - total > 0x1000000) ? 0x1000000 : (unsigned) (len - total) ;
+
 		current = flac_read_loop (psf, readlen) ;
 		if (current == 0)
 			break ;
@@ -937,10 +1029,10 @@ flac_read_flac2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 static sf_count_t
 flac_write_s2flac (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 {	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
-	void (*convert) (const short *, FLAC__int32 *, int) ;
+	void (*convert) (const short *, int32_t *, int) ;
 	int bufferlen, writecount, thiswrite ;
 	sf_count_t	total = 0 ;
-	FLAC__int32* buffer = pflac->encbuffer ;
+	int32_t* buffer = pflac->encbuffer ;
 
 	switch (SF_CODEC (psf->sf.format))
 	{	case SF_FORMAT_PCM_S8 :
@@ -956,13 +1048,13 @@ flac_write_s2flac (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 			return -1 ;
 		} ;
 
-	bufferlen = ENC_BUFFER_SIZE / (sizeof (FLAC__int32) * psf->sf.channels) ;
+	bufferlen = ENC_BUFFER_SIZE / (sizeof (int32_t) * psf->sf.channels) ;
 	bufferlen *= psf->sf.channels ;
 
 	while (len > 0)
 	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
 		convert (ptr + total, buffer, writecount) ;
-		if (FLAC__stream_encoder_process_interleaved (pflac->fse, buffer, writecount/psf->sf.channels))
+		if (FLAC__stream_encoder_process_interleaved (pflac->fse, buffer, writecount / psf->sf.channels))
 			thiswrite = writecount ;
 		else
 			break ;
@@ -979,10 +1071,10 @@ flac_write_s2flac (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 static sf_count_t
 flac_write_i2flac (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 {	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
-	void (*convert) (const int *, FLAC__int32 *, int) ;
+	void (*convert) (const int *, int32_t *, int) ;
 	int bufferlen, writecount, thiswrite ;
 	sf_count_t	total = 0 ;
-	FLAC__int32* buffer = pflac->encbuffer ;
+	int32_t* buffer = pflac->encbuffer ;
 
 	switch (SF_CODEC (psf->sf.format))
 	{	case SF_FORMAT_PCM_S8 :
@@ -998,13 +1090,13 @@ flac_write_i2flac (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 			return -1 ;
 		} ;
 
-	bufferlen = ENC_BUFFER_SIZE / (sizeof (FLAC__int32) * psf->sf.channels) ;
+	bufferlen = ENC_BUFFER_SIZE / (sizeof (int32_t) * psf->sf.channels) ;
 	bufferlen *= psf->sf.channels ;
 
 	while (len > 0)
 	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
 		convert (ptr + total, buffer, writecount) ;
-		if (FLAC__stream_encoder_process_interleaved (pflac->fse, buffer, writecount/psf->sf.channels))
+		if (FLAC__stream_encoder_process_interleaved (pflac->fse, buffer, writecount / psf->sf.channels))
 			thiswrite = writecount ;
 		else
 			break ;
@@ -1021,10 +1113,10 @@ flac_write_i2flac (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 static sf_count_t
 flac_write_f2flac (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 {	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
-	void (*convert) (const float *, FLAC__int32 *, int, int) ;
+	void (*convert) (const float *, int32_t *, int, int) ;
 	int bufferlen, writecount, thiswrite ;
 	sf_count_t	total = 0 ;
-	FLAC__int32* buffer = pflac->encbuffer ;
+	int32_t* buffer = pflac->encbuffer ;
 
 	switch (SF_CODEC (psf->sf.format))
 	{	case SF_FORMAT_PCM_S8 :
@@ -1040,13 +1132,13 @@ flac_write_f2flac (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 			return -1 ;
 		} ;
 
-	bufferlen = ENC_BUFFER_SIZE / (sizeof (FLAC__int32) * psf->sf.channels) ;
+	bufferlen = ENC_BUFFER_SIZE / (sizeof (int32_t) * psf->sf.channels) ;
 	bufferlen *= psf->sf.channels ;
 
 	while (len > 0)
 	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
 		convert (ptr + total, buffer, writecount, psf->norm_float) ;
-		if (FLAC__stream_encoder_process_interleaved (pflac->fse, buffer, writecount/psf->sf.channels))
+		if (FLAC__stream_encoder_process_interleaved (pflac->fse, buffer, writecount / psf->sf.channels))
 			thiswrite = writecount ;
 		else
 			break ;
@@ -1061,7 +1153,7 @@ flac_write_f2flac (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 } /* flac_write_f2flac */
 
 static void
-f2flac8_clip_array (const float *src, FLAC__int32 *dest, int count, int normalize)
+f2flac8_clip_array (const float *src, int32_t *dest, int count, int normalize)
 {	float normfact, scaled_value ;
 
 	normfact = normalize ? (8.0 * 0x10) : 1.0 ;
@@ -1083,28 +1175,27 @@ f2flac8_clip_array (const float *src, FLAC__int32 *dest, int count, int normaliz
 } /* f2flac8_clip_array */
 
 static void
-f2flac16_clip_array (const float *src, FLAC__int32 *dest, int count, int normalize)
-{
-  float normfact, scaled_value ;
+f2flac16_clip_array (const float *src, int32_t *dest, int count, int normalize)
+{	float normfact, scaled_value ;
 
-  normfact = normalize ? (8.0 * 0x1000) : 1.0 ;
+	normfact = normalize ? (8.0 * 0x1000) : 1.0 ;
 
-  while (--count >= 0) {
-    scaled_value = src [count] * normfact ;
-    if (CPU_CLIPS_POSITIVE == 0 && scaled_value >= (1.0 * 0x7FFF)) {
-      dest [count] = 0x7FFF ;
-      continue ;
-    }
-    if (CPU_CLIPS_NEGATIVE == 0 && scaled_value <= (-8.0 * 0x1000)) {
-      dest [count] = 0x8000 ;
-      continue ;
-    }
-    dest [count] = lrintf (scaled_value) ;
-  }
+	while (--count >= 0)
+	{	scaled_value = src [count] * normfact ;
+		if (CPU_CLIPS_POSITIVE == 0 && scaled_value >= (1.0 * 0x7FFF))
+		{	dest [count] = 0x7FFF ;
+			continue ;
+			} ;
+		if (CPU_CLIPS_NEGATIVE == 0 && scaled_value <= (-8.0 * 0x1000))
+		{	dest [count] = 0x8000 ;
+			continue ;
+			} ;
+		dest [count] = lrintf (scaled_value) ;
+		} ;
 } /* f2flac16_clip_array */
 
 static void
-f2flac24_clip_array (const float *src, FLAC__int32 *dest, int count, int normalize)
+f2flac24_clip_array (const float *src, int32_t *dest, int count, int normalize)
 {	float normfact, scaled_value ;
 
 	normfact = normalize ? (8.0 * 0x100000) : 1.0 ;
@@ -1127,7 +1218,7 @@ f2flac24_clip_array (const float *src, FLAC__int32 *dest, int count, int normali
 } /* f2flac24_clip_array */
 
 static void
-f2flac8_array (const float *src, FLAC__int32 *dest, int count, int normalize)
+f2flac8_array (const float *src, int32_t *dest, int count, int normalize)
 {	float normfact = normalize ? (1.0 * 0x7F) : 1.0 ;
 
 	while (--count >= 0)
@@ -1135,7 +1226,7 @@ f2flac8_array (const float *src, FLAC__int32 *dest, int count, int normalize)
 } /* f2flac8_array */
 
 static void
-f2flac16_array (const float *src, FLAC__int32 *dest, int count, int normalize)
+f2flac16_array (const float *src, int32_t *dest, int count, int normalize)
 {	float normfact = normalize ? (1.0 * 0x7FFF) : 1.0 ;
 
 	while (--count >= 0)
@@ -1143,7 +1234,7 @@ f2flac16_array (const float *src, FLAC__int32 *dest, int count, int normalize)
 } /* f2flac16_array */
 
 static void
-f2flac24_array (const float *src, FLAC__int32 *dest, int count, int normalize)
+f2flac24_array (const float *src, int32_t *dest, int count, int normalize)
 {	float normfact = normalize ? (1.0 * 0x7FFFFF) : 1.0 ;
 
 	while (--count >= 0)
@@ -1153,10 +1244,10 @@ f2flac24_array (const float *src, FLAC__int32 *dest, int count, int normalize)
 static sf_count_t
 flac_write_d2flac (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 {	FLAC_PRIVATE* pflac = (FLAC_PRIVATE*) psf->codec_data ;
-	void (*convert) (const double *, FLAC__int32 *, int, int) ;
+	void (*convert) (const double *, int32_t *, int, int) ;
 	int bufferlen, writecount, thiswrite ;
 	sf_count_t	total = 0 ;
-	FLAC__int32* buffer = pflac->encbuffer ;
+	int32_t* buffer = pflac->encbuffer ;
 
 	switch (SF_CODEC (psf->sf.format))
 	{	case SF_FORMAT_PCM_S8 :
@@ -1172,13 +1263,13 @@ flac_write_d2flac (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 			return -1 ;
 		} ;
 
-	bufferlen = ENC_BUFFER_SIZE / (sizeof (FLAC__int32) * psf->sf.channels) ;
+	bufferlen = ENC_BUFFER_SIZE / (sizeof (int32_t) * psf->sf.channels) ;
 	bufferlen *= psf->sf.channels ;
 
 	while (len > 0)
 	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
 		convert (ptr + total, buffer, writecount, psf->norm_double) ;
-		if (FLAC__stream_encoder_process_interleaved (pflac->fse, buffer, writecount/psf->sf.channels))
+		if (FLAC__stream_encoder_process_interleaved (pflac->fse, buffer, writecount / psf->sf.channels))
 			thiswrite = writecount ;
 		else
 			break ;
@@ -1193,7 +1284,7 @@ flac_write_d2flac (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 } /* flac_write_d2flac */
 
 static void
-d2flac8_clip_array (const double *src, FLAC__int32 *dest, int count, int normalize)
+d2flac8_clip_array (const double *src, int32_t *dest, int count, int normalize)
 {	double normfact, scaled_value ;
 
 	normfact = normalize ? (8.0 * 0x10) : 1.0 ;
@@ -1215,7 +1306,7 @@ d2flac8_clip_array (const double *src, FLAC__int32 *dest, int count, int normali
 } /* d2flac8_clip_array */
 
 static void
-d2flac16_clip_array (const double *src, FLAC__int32 *dest, int count, int normalize)
+d2flac16_clip_array (const double *src, int32_t *dest, int count, int normalize)
 {	double normfact, scaled_value ;
 
 	normfact = normalize ? (8.0 * 0x1000) : 1.0 ;
@@ -1237,7 +1328,7 @@ d2flac16_clip_array (const double *src, FLAC__int32 *dest, int count, int normal
 } /* d2flac16_clip_array */
 
 static void
-d2flac24_clip_array (const double *src, FLAC__int32 *dest, int count, int normalize)
+d2flac24_clip_array (const double *src, int32_t *dest, int count, int normalize)
 {	double normfact, scaled_value ;
 
 	normfact = normalize ? (8.0 * 0x100000) : 1.0 ;
@@ -1259,7 +1350,7 @@ d2flac24_clip_array (const double *src, FLAC__int32 *dest, int count, int normal
 } /* d2flac24_clip_array */
 
 static void
-d2flac8_array (const double *src, FLAC__int32 *dest, int count, int normalize)
+d2flac8_array (const double *src, int32_t *dest, int count, int normalize)
 {	double normfact = normalize ? (1.0 * 0x7F) : 1.0 ;
 
 	while (--count >= 0)
@@ -1267,7 +1358,7 @@ d2flac8_array (const double *src, FLAC__int32 *dest, int count, int normalize)
 } /* d2flac8_array */
 
 static void
-d2flac16_array (const double *src, FLAC__int32 *dest, int count, int normalize)
+d2flac16_array (const double *src, int32_t *dest, int count, int normalize)
 {	double normfact = normalize ? (1.0 * 0x7FFF) : 1.0 ;
 
 	while (--count >= 0)
@@ -1275,7 +1366,7 @@ d2flac16_array (const double *src, FLAC__int32 *dest, int count, int normalize)
 } /* d2flac16_array */
 
 static void
-d2flac24_array (const double *src, FLAC__int32 *dest, int count, int normalize)
+d2flac24_array (const double *src, int32_t *dest, int count, int normalize)
 {	double normfact = normalize ? (1.0 * 0x7FFFFF) : 1.0 ;
 
 	while (--count >= 0)
@@ -1297,12 +1388,19 @@ flac_seek (SF_PRIVATE *psf, int UNUSED (mode), sf_count_t offset)
 	pflac->frame = NULL ;
 
 	if (psf->file.mode == SFM_READ)
-	{	FLAC__uint64 position ;
-		if (FLAC__stream_decoder_seek_absolute (pflac->fsd, offset))
-		{	FLAC__stream_decoder_get_decode_position (pflac->fsd, &position) ;
+	{	if (FLAC__stream_decoder_seek_absolute (pflac->fsd, offset))
+			return offset ;
+
+		if (offset == psf->sf.frames)
+		{	/*
+			** If we've been asked to seek to the very end of the file, libFLAC
+			** will return an error. However, we know the length of the file so
+			** instead of returning an error, we can return the offset.
+			*/
 			return offset ;
 			} ;
 
+		psf->error = SFE_BAD_SEEK ;
 		return ((sf_count_t) -1) ;
 		} ;
 
@@ -1312,7 +1410,17 @@ flac_seek (SF_PRIVATE *psf, int UNUSED (mode), sf_count_t offset)
 	return ((sf_count_t) -1) ;
 } /* flac_seek */
 
-#else /* HAVE_EXTERNAL_LIBS */
+static int
+flac_byterate (SF_PRIVATE *psf)
+{
+	if (psf->file.mode == SFM_READ)
+		return (psf->datalength * psf->sf.samplerate) / psf->sf.frames ;
+
+	return -1 ;
+} /* flac_byterate */
+
+
+#else /* HAVE_EXTERNAL_XIPH_LIBS */
 
 int
 flac_open	(SF_PRIVATE *psf)
