@@ -422,7 +422,7 @@ wavlike_read_fmt_chunk (SF_PRIVATE *psf, int fmtsize)
 						psf_log_printf (psf, "  Extra Bytes   : %d (should be 2)\n", wav_fmt->g72x.extrabytes) ;
 					else
 						psf_log_printf (psf, "  Extra Bytes   : %d\n", wav_fmt->g72x.extrabytes) ;
-						psf_log_printf (psf, "  Aux Blk Size  : %d\n", wav_fmt->g72x.auxblocksize) ;
+					psf_log_printf (psf, "  Aux Blk Size  : %d\n", wav_fmt->g72x.auxblocksize) ;
 					}
 				else if (fmtsize == 18)
 				{	bytesread += psf_binheader_readf (psf, "2", &(wav_fmt->g72x.extrabytes)) ;
@@ -895,12 +895,10 @@ wavlike_write_cart_chunk (SF_PRIVATE *psf)
 int
 wavlike_subchunk_parse (SF_PRIVATE *psf, int chunk, uint32_t chunk_length)
 {	sf_count_t	current_pos ;
-	char		buffer [512] ;
-	uint32_t 	chunk_size, bytesread ;
+	char		buffer [2048] ;
+	uint32_t 	chunk_size, bytesread = 0 ;
 
-	current_pos = psf_fseek (psf, 0, SEEK_CUR) - 4 ;
-
-	bytesread = sizeof (chunk_length) ;
+	current_pos = psf_fseek (psf, 0, SEEK_CUR) ;
 
 	if (chunk_length <= 8)
 	{	/* This case is for broken files generated by PEAK. */
@@ -910,12 +908,6 @@ wavlike_subchunk_parse (SF_PRIVATE *psf, int chunk, uint32_t chunk_length)
 		return 0 ;
 		} ;
 
-	if (psf->headindex + chunk_length > SIGNED_SIZEOF (psf->header))
-	{	psf_log_printf (psf, "%M : %u (too long)\n", chunk, chunk_length) ;
-		psf_binheader_readf (psf, "j", chunk_length) ;
-		return 0 ;
-		} ;
-
 	if (current_pos + chunk_length > psf->filelength)
 	{	psf_log_printf (psf, "%M : %u (should be %d)\n", chunk, chunk_length, (int) (psf->filelength - current_pos)) ;
 		chunk_length = psf->filelength - current_pos ;
@@ -983,8 +975,7 @@ wavlike_subchunk_parse (SF_PRIVATE *psf, int chunk, uint32_t chunk_length)
 					chunk_size += (chunk_size & 1) ;
 					if (chunk_size >= SIGNED_SIZEOF (buffer) || chunk_size >= chunk_length)
 					{	psf_log_printf (psf, "  *** %M : %u (too big)\n", chunk, chunk_size) ;
-						psf_binheader_readf (psf, "j", chunk_length) ;
-						return 0 ;
+						goto cleanup_subchunk_parse ;
 						} ;
 
 					bytesread += psf_binheader_readf (psf, "b", buffer, chunk_size) ;
@@ -1000,8 +991,7 @@ wavlike_subchunk_parse (SF_PRIVATE *psf, int chunk, uint32_t chunk_length)
 						chunk_size += (chunk_size & 1) ;
 						if (chunk_size < 1 || chunk_size >= SIGNED_SIZEOF (buffer) || chunk_size >= chunk_length)
 						{	psf_log_printf (psf, "  *** %M : %u (too big)\n", chunk, chunk_size) ;
-							psf_binheader_readf (psf, "j", chunk_length) ;
-							break ;
+							goto cleanup_subchunk_parse ;
 							} ;
 
 						bytesread += psf_binheader_readf (psf, "b", buffer, chunk_size) ;
@@ -1018,13 +1008,11 @@ wavlike_subchunk_parse (SF_PRIVATE *psf, int chunk, uint32_t chunk_length)
 					chunk_size += (chunk_size & 1) ;
 					if (chunk_size >= SIGNED_SIZEOF (buffer) || chunk_size >= chunk_length)
 					{	psf_log_printf (psf, "  *** %M : %u (too big)\n", chunk, chunk_size) ;
-						psf_binheader_readf (psf, "j", chunk_length) ;
-						return 0 ;
+						goto cleanup_subchunk_parse ;
 						} ;
 
-					bytesread += psf_binheader_readf (psf, "j", chunk_size) ;
 					psf_log_printf (psf, "    %M : %u\n", chunk, chunk_size) ;
-					break ;
+					goto cleanup_subchunk_parse ;
 
 			default :
 					bytesread += psf_binheader_readf (psf, "4", &chunk_size) ;
@@ -1073,10 +1061,10 @@ wavlike_subchunk_parse (SF_PRIVATE *psf, int chunk, uint32_t chunk_length)
 			} ;
 		} ;
 
-	current_pos = psf_fseek (psf, 0, SEEK_CUR) - current_pos ;
+cleanup_subchunk_parse :
 
-	if (current_pos - 4 != chunk_length)
-		psf_log_printf (psf, "**** Bad chunk length %d sbould be %D\n", chunk_length, current_pos - 4) ;
+	if (chunk_length > bytesread)
+		bytesread += psf_binheader_readf (psf, "j", chunk_length - bytesread) ;
 
 	return 0 ;
 } /* wavlike_subchunk_parse */
@@ -1088,7 +1076,7 @@ wavlike_write_strings (SF_PRIVATE *psf, int location)
 	if (psf_location_string_count (psf, location) == 0)
 		return ;
 
-	prev_head_index = psf->headindex + 4 ;
+	prev_head_index = psf->header.indx + 4 ;
 
 	psf_binheader_writef (psf, "m4m", LIST_MARKER, 0xBADBAD, INFO_MARKER) ;
 
@@ -1140,10 +1128,10 @@ wavlike_write_strings (SF_PRIVATE *psf, int location)
 			} ;
 		} ;
 
-	saved_head_index = psf->headindex ;
-	psf->headindex = prev_head_index ;
+	saved_head_index = psf->header.indx ;
+	psf->header.indx = prev_head_index ;
 	psf_binheader_writef (psf, "4", saved_head_index - prev_head_index - 4) ;
-	psf->headindex = saved_head_index ;
+	psf->header.indx = saved_head_index ;
 
 } /* wavlike_write_strings */
 
