@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2014 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include <math.h>
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#include <float_cast.h>
-
 #include <sndfile.h>
 
 #include "utils.h"
@@ -62,26 +62,22 @@ dwvw_test (const char *filename, int format, int bit_width)
 	srand (123456) ;
 
 	/* Only want to grab the top bit_width bits. */
-	bit_mask = (-1 << (32 - bit_width)) ;
+	bit_mask = arith_shift_left (-1, 32 - bit_width) ;
 
 	print_test_name ("dwvw_test", filename) ;
 
-	sfinfo.format		= format ;
-	sfinfo.samplerate	= 44100 ;
-	sfinfo.frames		= -1 ; /* Unknown! */
-	sfinfo.channels		= 1 ;
+	sf_info_setup (&sfinfo, format, 44100, 1) ;
 
 	file = test_open_file_or_die (filename, SFM_WRITE, &sfinfo, SF_TRUE, __LINE__) ;
 
 	/* Generate random.frames. */
-	k = 0 ;
 	for (k = 0 ; k < BUFFER_SIZE / 2 ; k++)
 	{	value = 0x7FFFFFFF * sin (123.0 / sfinfo.samplerate * 2 * k * M_PI) ;
 		write_buf [k] = bit_mask & lrint (value) ;
 		} ;
 
 	for ( ; k < BUFFER_SIZE ; k++)
-		write_buf [k] = bit_mask & ((rand () << 11) ^ (rand () >> 11)) ;
+		write_buf [k] = bit_mask & (arith_shift_left (rand (), 11) ^ (rand () >> 11)) ;
 
 	sf_write_int (file, write_buf, BUFFER_SIZE) ;
 	sf_close (file) ;
@@ -110,10 +106,4 @@ dwvw_test (const char *filename, int format, int bit_width)
 
 	return ;
 } /* dwvw_test */
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 9e7731c3-43cd-4de5-b539-1ce6fa11d937
-*/
+
