@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2001-2016 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software ; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <inttypes.h>
 
 #include <sys/stat.h>
 #include <math.h>
@@ -43,13 +44,14 @@
 
 #include	"utils.h"
 
-#define	BUFFER_LEN		(1<<10)
+#define	BUFFER_LEN		(1 << 10)
 #define LOG_BUFFER_SIZE	1024
 
 static void	zero_data_test (const char *filename, int format) ;
 static void	filesystem_full_test (int format) ;
 static void	permission_test (const char *filename, int typemajor) ;
 static void	wavex_amb_test (const char *filename) ;
+static void rf64_downgrade_test (const char *filename) ;
 
 int
 main (int argc, char *argv [])
@@ -65,7 +67,7 @@ main (int argc, char *argv [])
 		exit (1) ;
 		} ;
 
-	do_all=!strcmp (argv [1], "all") ;
+	do_all = ! strcmp (argv [1], "all") ;
 
 	if (do_all || ! strcmp (argv [1], "wav"))
 	{	zero_data_test ("zerolen.wav", SF_FORMAT_WAV | SF_FORMAT_PCM_16) ;
@@ -139,9 +141,10 @@ main (int argc, char *argv [])
 		} ;
 
 	if (do_all || ! strcmp (argv [1], "rf64"))
-	{	zero_data_test ("zerolen.rf64", SF_FORMAT_W64 | SF_FORMAT_PCM_16) ;
-		filesystem_full_test (SF_FORMAT_W64 | SF_FORMAT_PCM_16) ;
-		permission_test ("readonly.rf64", SF_FORMAT_W64) ;
+	{	zero_data_test ("zerolen.rf64", SF_FORMAT_RF64 | SF_FORMAT_PCM_16) ;
+		filesystem_full_test (SF_FORMAT_RF64 | SF_FORMAT_PCM_16) ;
+		permission_test ("readonly.rf64", SF_FORMAT_RF64) ;
+		rf64_downgrade_test ("downgrade.wav") ;
 		test_count++ ;
 		} ;
 
@@ -220,11 +223,10 @@ static void
 zero_data_test (const char *filename, int format)
 {	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
-	int			frames ;
 
 	switch (format & SF_FORMAT_TYPEMASK)
 	{	case SF_FORMAT_OGG :
-			if (HAVE_EXTERNAL_LIBS == 0)
+			if (HAVE_EXTERNAL_XIPH_LIBS == 0)
 				return ;
 			break ;
 		default :
@@ -238,8 +240,6 @@ zero_data_test (const char *filename, int format)
 	sfinfo.channels = 1 ;
 	sfinfo.frames = 0 ;
 
-	frames = BUFFER_LEN / sfinfo.channels ;
-
 	file = test_open_file_or_die (filename, SFM_WRITE, &sfinfo, SF_TRUE, __LINE__) ;
 
 	sf_close (file) ;
@@ -261,7 +261,6 @@ filesystem_full_test (int format)
 	struct stat buf ;
 
 	const char	*filename = "/dev/full", *errorstr ;
-	int			frames ;
 
 #if (defined (WIN32) || defined (_WIN32))
 	/* Can't run this test on Win32 so return. */
@@ -288,8 +287,6 @@ filesystem_full_test (int format)
 	sfinfo.channels = 1 ;
 	sfinfo.frames = 0 ;
 
-	frames = BUFFER_LEN / sfinfo.channels ;
-
 	if ((file = sf_open (filename, SFM_WRITE, &sfinfo)) != NULL)
 	{	printf ("\n\nLine %d : Error, file should not have openned.\n", __LINE__ - 1) ;
 		exit (1) ;
@@ -321,7 +318,6 @@ permission_test (const char *filename, int typemajor)
 	SNDFILE		*file ;
 	SF_INFO		sfinfo ;
 	const char	*errorstr ;
-	int			frames ;
 
 	/* Make sure errno is zero before doing anything else. */
 	errno = 0 ;
@@ -360,8 +356,6 @@ permission_test (const char *filename, int typemajor)
 	sfinfo.channels = 1 ;
 	sfinfo.frames = 0 ;
 
-	frames = BUFFER_LEN / sfinfo.channels ;
-
 	if ((file = sf_open (filename, SFM_WRITE, &sfinfo)) != NULL)
 	{	printf ("\n\nLine %d : Error, file should not have opened.\n", __LINE__ - 1) ;
 		exit (1) ;
@@ -422,3 +416,58 @@ wavex_amb_test (const char *filename)
 	unlink (filename) ;
 	puts ("ok") ;
 } /* wavex_amb_test */
+
+static void
+rf64_downgrade_test (const char *filename)
+{	static short	output	[BUFFER_LEN] ;
+	static short	input	[BUFFER_LEN] ;
+
+	SNDFILE		*file ;
+	SF_INFO		sfinfo ;
+	unsigned	k ;
+
+	print_test_name (__func__, filename) ;
+
+	sf_info_clear (&sfinfo) ;
+
+	sfinfo.samplerate	= 44100 ;
+	sfinfo.frames		= ARRAY_LEN (output) ;
+	sfinfo.channels		= 1 ;
+	sfinfo.format		= SF_FORMAT_RF64 | SF_FORMAT_PCM_16 ;
+
+	file = test_open_file_or_die (filename, SFM_WRITE, &sfinfo, SF_TRUE, __LINE__) ;
+
+	exit_if_true (sf_command (file, SFC_RF64_AUTO_DOWNGRADE, NULL, SF_FALSE) != SF_FALSE, "\n\nLine %d: sf_command failed.\n", __LINE__) ;
+	exit_if_true (sf_command (file, SFC_RF64_AUTO_DOWNGRADE, NULL, SF_TRUE) != SF_TRUE, "\n\nLine %d: sf_command failed.\n", __LINE__) ;
+
+	test_write_short_or_die (file, 0, output, ARRAY_LEN (output), __LINE__) ;
+
+	exit_if_true (sf_command (file, SFC_RF64_AUTO_DOWNGRADE, NULL, SF_FALSE) != SF_TRUE, "\n\nLine %d: sf_command failed.\n", __LINE__) ;
+	exit_if_true (sf_command (file, SFC_RF64_AUTO_DOWNGRADE, NULL, SF_TRUE) != SF_TRUE, "\n\nLine %d: sf_command failed.\n", __LINE__) ;
+
+	sf_close (file) ;
+
+	memset (input, 0, sizeof (input)) ;
+	sf_info_clear (&sfinfo) ;
+
+	file = test_open_file_or_die (filename, SFM_READ, &sfinfo, SF_TRUE, __LINE__) ;
+
+	exit_if_true (sfinfo.format != (SF_FORMAT_WAVEX | SF_FORMAT_PCM_16), "\n\nLine %d: RF64 to WAV downgrade failed.\n", __LINE__) ;
+	exit_if_true (sfinfo.frames != ARRAY_LEN (output), "\n\nLine %d: Incorrect number of frames in file (too short). (%d should be %d)\n", __LINE__, (int) sfinfo.frames, (int) ARRAY_LEN (output)) ;
+	exit_if_true (sfinfo.channels != 1, "\n\nLine %d: Incorrect number of channels in file.\n", __LINE__) ;
+
+	check_log_buffer_or_die (file, __LINE__) ;
+
+	test_read_short_or_die (file, 0, input, ARRAY_LEN (input), __LINE__) ;
+
+	sf_close (file) ;
+
+	for (k = 0 ; k < ARRAY_LEN (input) ; k++)
+		exit_if_true (input [k] != output [k],
+			"\n\nLine: %d: Error on input %d, expected %d, got %d\n", __LINE__, k, output [k], input [k]) ;
+
+	puts ("ok") ;
+	unlink (filename) ;
+
+	return ;
+} /* rf64_downgrade_test */
