@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2001-2012 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,12 +26,12 @@
 #include <stdlib.h>
 #include <string.h>
 
-#if (OS_IS_WIN32)
+#if (OS_IS_WIN32 || defined __OS2__ || HAVE_PIPE == 0 || HAVE_WAITPID == 0)
 
 int
 main (void)
 {
-	puts ("    pipe_test  : this test doesn't work on win32.") ;
+	puts ("    pipe_test  : this test doesn't work on this OS.") ;
 	return 0 ;
 } /* main */
 
